/***********************************************************
 * $Id: fancysockpacketdevice.h 1244 2014-05-29 21:13:55Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** A base class, which wraps TCP/IP sockets for use with libxml.
** Extensible for using SSL or socks etc. by using many virtual functions.
**
*****************************************************************************/

#ifndef FANCYSOCK_PACKET_DEVICE_H
#define FANCYSOCK_PACKET_DEVICE_H

#include <fancypollablechannel.h>
#include <fancysockaddress.h>

namespace Fancy {
namespace Sock {

/** A base class for packet sockets.

    This class defines an socket device, which is represented by
    a POSIX filedecriptor as returned by socket(2).

    It may be reimplemented in order to use many other socket encapsulations
    such as SSL or SOCKS.
  */
  class FANCYSOCK_API PacketDevice : virtual public Fancy::PollableChannel
  {
  public:
	  typedef Fancy::ObjPtr<PacketDevice> ptr_t;

	  /**
	   * Instantiate a new ICMP instance.
       *
       * This method will use close_on_exec=true
	   */
	  static ptr_t newICMPInstance();

      /**
       * Instantiate a new ICMP instance.
       *
       * @param close_on_exec Whether the socket will be closed when any of the
       *                      exec*() functions is used to run another program,
       *                      e.g. in a forked instance of this process.
       *                      This flag is ignored under Windows.
       */
      static ptr_t newICMPInstance(bool close_on_exec);

      /**
       * Instantiate a new UDP instance for client use.
       *
       * This method will use close_on_exec=true
       */
      static ptr_t newUDPInstance();

      /**
       * Instantiate a new UDP instance for client use.
       *
       * @param close_on_exec Whether the socket will be closed when any of the
       *                      exec*() functions is used to run another program,
       *                      e.g. in a forked instance of this process.
       *                      This flag is ignored under Windows.
       */
      static ptr_t newUDPInstance(bool close_on_exec);

      /**
       * Instantiate and bind a new UDP instance and bind it to the given address.
       *
       * This method will use close_on_exec=true
       */
      static ptr_t bindUDP (const Address::ptr_t &addr);

      /**
       * Instantiate and bind a new UDP instance and bind it to the given address.
       *
       * @param close_on_exec Whether the socket will be closed when any of the
       *                      exec*() functions is used to run another program,
       *                      e.g. in a forked instance of this process.
       *                      This flag is ignored under Windows.
       */
      static ptr_t bindUDP (const Address::ptr_t &addr, bool close_on_exec);

     /**
	   * Receive a datagram and return the number of bytes received.
	   * The addr pointer is set to the origin of the datagram.
	   */
	  virtual int recvFrom(void *buf, int len, Address::ptr_t &addr) = 0;

	  /**
	   * Send a datagram to the given address.
	   */
	  virtual int sendTo(const void *buf, int len, const Address::ptr_t &addr) = 0;

  protected:
	  PacketDevice();
	  virtual ~PacketDevice();
  };


  }
}

#endif // ! FANCYSOCK_PACKET_DEVICE_H
